

%---------------------------------------------
%  coreference.pl
%
%  coreference resolution by attribute similarity 
%  Luc Russell
%
%  cluster/2 is the main predicate,
%  it reads the files specified in readGATE.pl
%  and returns 1. the posSent/3 representation 
%  of the GATE file, 2. a list of class/3:
%  class(SentenceNo, NounClass, Token)
%  SentenceNo = GATE sentence number
%  NounClass  = an integer representing a class
%               of nouns
%  Token      = a token, tagged as a noun
%
%  There may be a one-to-many mapping from 
%  NounClass to Token. Each NounClass maps to
%  coreferring nouns. 
%
%  coreference-all.pl requires readGATE-new.pl
%
% coreference requires: 
% abbreviation_in_ontology/1
% abbrev_class_in_ontology/2
% class_superclass_in_ontology/2
% superclass_in_ontology/1


%
%---------------------------------------------



%Representation builder
%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% repBuilder.pl
%% Take output file from GATE and an input document
%% and build a list of attributes for each noun in a text
%%
%%

%%%%%%%%%%%%%%%%%%%%%%%%%
:- dynamic count/1.
%:- consult(chemontology).
%:- consult(namedata).
%:- consult(wordnet/prologwn/wn_s).
%:- consult(wordnet/prologwn/wn_hyp).
%%%%%%%%%%%%%%%%%%%%%%%%%
%% TOP LEVEL
%%
%%


buildRep(E,R,Next2):-          % renamed by SA; previously called processGate
	resetCount,
	filterOut(E,F),
	changeCase(F,R),
	getWords(R,S),
	getNumber(S,T),
	defCheck(T,U),
	getPronounType(U,M),
	nameTest(M, G),
        getMax(G,W),
	addMaxtoRep(W,G,H),
	getAppositive(H,V),
	getClass(V,Next),
	getSuperClass(Next, Next2). 

%%
%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% initialise the counter predicate
%%
resetCount:-
	retractall(count(_)),
	assertz(count(1)).
%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Get nouns from output, discard other parts of speech, and count
% nouns for value of UniqueId
%

getWords([], []):-!.
getWords([posSent(SentNo, UniqueId, POS, Word)|A],
	[posSent(SentNo, UniqueId, POS, Word)|S]):-
	nounClass(POS),
	getWords(A,S).
getWords([posSent(SentNo, UniqueId, POS, Word)|A],
	[posSent(SentNo, UniqueId, POS, Word)|S]):-
	detClass(Word),
	getWords(A,S).
getWords([posSent(SentNo, UniqueId, POS, Word)|A],
	[posSent(SentNo, UniqueId, POS, Word)|S]):-
	POS = 'comma' ,
	getWords(A,S).
getWords([posSent(S, UID, P, Word)|X],
	[posSent(S, UID, P, Word)|R]):-
	firstperson(Word),
	getWords(X,R).
getWords([posSent(S, UID, P, Word)|X],
	[posSent(S, UID, P, Word)|R]):-
	sndperson(Word),
	getWords(X,R).

getWords([posSent(S, UID, P, Word)|X],
	[posSent(S, UID, P, Word)|R]):-
	thrdperson(Word),
	getWords(X,R).
%Word is a chemical symbol
getWords([posSent(S, UID, P, Word)|X],
	[posSent(S, UID, P, Word)|R]):-
	abbreviation_in_ontology(Word),
	getWords(X,R).
getWords([posSent(_, _, _, _)|B], C):-!,
	getWords(B,C).
nounClass('nn').
nounClass('nnp').
nounClass('nnps').
nounClass('nns').
nounClass('np').
nounClass('nps').
detClass('a').
detClass('an').
detClass('the').
%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%
%
% Start building a list of attributes for each noun
%
getNumber([], []):-!.
% test if Part of speech is singular
getNumber([posSent(SentNo, UniqueID, Pos, Word)|A],
	[posSent(SentNo, UniqueID, Pos, Word,[singpl(singular)])|R]):-
	singular(Pos),
	getNumber(A,R).
% test if word is singular (pronouns are not tagged for number)

getNumber([posSent(SNo, UID, P, W)|A], [posSent(SNo, UID, P,
	W,[singpl(singular)])|R]):-
	singular(W),
	getNumber(A,R).

% test if noun is plural
getNumber([posSent(SentNo, UniqueID, Pos, Word)|A],
	[posSent(SentNo, UniqueID,Pos, Word,[singpl(plural)])|R]):-
	plural(Pos),
	getNumber(A,R).
% test if noun is neither singular or plural
getNumber([posSent(SentNo, UniqueID, Pos, Word)|A],
	[posSent(SentNo, UniqueID,Pos, Word,[singpl(none)])|R]):-
	getNumber(A,R).
singular('nn').
singular('nnp').
singular('np').
plural('nnps').
plural('nns').
plural('nps').
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% nameTest/2
%%
%% Check namedata databases to check if word is a name
%%
nameTest([],[]):-!.
% Two consecutive words form one name; return the second word only

nameTest([posSent(_, _, _, Word, [_,_,_]),
	posSent(SentNo2, UniqueID2, Pos2, Word2,[A2,B2,C2])|L],
	[posSent(SentNo2, UniqueID2, Pos2, Word2, [A2,B2,C2,name(Word2)])|R]):-
	name(Word),
	name(Word2),
	nameTest(L,R).
% Individual word is a name
nameTest([posSent(SentNo, UniqueID, Pos, Word,[A,B,C])|L],
	 [posSent(SentNo, UniqueID, Pos, Word, [A,B,C,
	name(Word)])|R]):-
	name(Word),
	nameTest(L,R).
nameTest([posSent(SentNo, UniqueID, Pos, Word,[A,B,C])|L],
	[posSent(SentNo, UniqueID, Pos, Word, [A,B,C,
	name(none)])|R]):-
	nameTest(L,R).
%%
%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Test for definiteness
%
defCheck([], []):-!.
%% test if noun is preceded by 'the' (definite)
defCheck( [posSent(_, _, _, 'the', [singpl(_)]),
	posSent(SentNo2, UniqueID2, Pos2, Noun,
	[singpl(SingPl2)])|A],
	[posSent(SentNo2, UniqueID2, Pos2, Noun,
	[singpl(SingPl2), article(definite)])|R] ):-
	defCheck(A,R).
%% test if noun is preceded by 'a' (indefinite)
defCheck( [posSent(_, _, _, 'a', [singpl(_)]),
	posSent(SentNo2, UniqueID2, Pos2, Noun,
	[singpl(SingPl2)])|A],
	[posSent(SentNo2, UniqueID2, Pos2, Noun,
	[singpl(SingPl2), article(indefinite)])|R] ):-
	defCheck(A,R).
%% test if noun is preceded by 'an' (indefinite)
defCheck( [posSent(_, _, _, 'an', [singpl(_)]),
	posSent(SentNo2, UniqueID2, Pos2, Noun,
	[singpl(SingPl2)])|A],
	[posSent(SentNo2, UniqueID2, Pos2, Noun,
	[singpl(SingPl2), article(indefinite)])|R] ):-
	defCheck(A,R).

%% test if noun does not have a definiteness value
defCheck( [posSent(SentNo, UniqueID, Pos, Word,
	[singpl(SingPl)])|A], [posSent(SentNo, UniqueID, Pos, Word,
	[singpl(SingPl), article(none)])|R] ):-
	defCheck(A,R).
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%
% getPronounType: mark pronouns as nominative, accusative, possessive
% or ambiguous. Other nouns get value none.
%

getPronounType([], []):- !.
%%% CASE 1: Word is first person pronoun
getPronounType( [posSent(S, U, P, W, [singpl(SP),
	article(D)])|A],
	[posSent(S, U, P, W, [singpl(SP), article(D),
	pronountype(fstper)])|R] ):-
	firstperson(W),
	getPronounType(A, R).
%%% CASE 2: Word is second person pronoun
getPronounType( [posSent(S, U, P, W, [singpl(SP),
	article(D)])|A],
	[posSent(S, U, P, W, [singpl(SP), article(D),
	pronountype(sndper)])|R]):-
	sndperson(W),
	getPronounType(A, R).
%%% CASE 3: Word is 3rd person pronoun
getPronounType( [posSent(S, U, P, W, [singpl(SP),
	article(D)])|A],
	[posSent(S, U, P, W, [singpl(SP), article(D),
	pronountype(thrdper)])|R]):-
	thrdperson(W),
	getPronounType(A, R).
%%% CASE 4: Word is not a pronoun
getPronounType( [posSent(S, U, P, W, [singpl(SP),
	article(D)])|A],
[posSent(S, U, P, W, [singpl(SP), article(D),
	pronountype(none)])|R]):-
	getPronounType(A, R).
%% Facts about pronoun type
firstperson('we').
firstperson('myself').
firstperson('my').
firstperson('mine').
firstperson('us').
firstperson('ourselves').
firstperson('our').
firstperson('ours').
sndperson('you').
sndperson('yourself').
sndperson('your').
sndperson('yours').
sndperson('yourselves').
thrdperson('he').
thrdperson('him').
thrdperson('himself').
thrdperson('his').
thrdperson('she').
thrdperson('her').
thrdperson('herself').
thrdperson('hers').
thrdperson('it').
thrdperson('itself').
thrdperson('its').
thrdperson('they').
thrdperson('themselves').
thrdperson('them').
thrdperson('their').
thrdperson('theirs').
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% getMax/2
%% Add the maximum UniqueId (total words in document) to the
%% NP representation
%%

getMax([posSent(_, Max, _, _, [_,_,_,_])], Max).
getMax([posSent(SentNo, X, Pos,Word, [A,B,C,D]),
	posSent(SentNo2, X2, Pos2,Word2, [A2,B2,C2,D2])|Rest], Max):-
	getMax([posSent(SentNo2, X2, Pos2, Word2,
	[A2,B2,C2,D2])|Rest], MaxRest),
	max(posSent(SentNo, X, Pos, Word, [A, B, C, D]),
	MaxRest, Max).
max(posSent(_, X, _, _, _), Y,X):-
	X >= Y.
max(posSent(_, X, _, _, _), Y,Y):-
	X < Y.
%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% addMaxtoRep/3
%% add Max value; replace SentNo value as no longer necessary
%%
addMaxtoRep(_,[],[]):-!.
addMaxtoRep(W, [posSent(SentNo, Ui, Pos, Word, [A,B,C,N])|D],
	[posSent([W,SentNo], Ui, Pos, Word, [A, B, C, N,
	position(Ui), word(Word)])|R]):-
	addMaxtoRep(W,D,R).
%%
%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% getAppositive/2
%%
%% Mark noun as appositive(true) if follows a noun phrase,
%	contains a determiner and is
%% surrounded by commas. Also filter out commas as no longer needed
%%
getAppositive([],[]):- !.

%% Case: input pattern is appositive
getAppositive([posSent(SN1, UI1, Pos1, Word1, [A,B,C,D,E,F]),
	%% Noun
posSent(_, _, _, Word2, _),
	%% Comma
posSent(_, _, _, Word3, _),
	%% Determiner
posSent(SN2, UI2, Pos4, Word4,
	[A1,B1,C1,D1,E1,F1]), %% Noun
posSent(_, _, _, Word5, _)|L],
	%% Comma
[posSent(SN1, UI1, Pos1, Word1, [A,B,C,D,E,F]),
	posSent(SN2, UI2, Pos4, Word4,
	[A1,B1,C1,D1,E1,F1, appositive(true)])|R]):-
	nounClass(Pos1),
	Word2 = ',',
	detClass(Word3),
	nounClass(Word4),
	Word5 = ',',
	getAppositive(L,R).
%% Filter out commas
getAppositive([posSent(_, _, comma, _, _)|L], R):-!,
	getAppositive(L,R).
getAppositive([posSent(SN1, UI1, Pos1, Word1,
	[A,B,C,D,E,F])|L],
	[posSent(SN1, UI1, Pos1, Word1, [A,B,C,D,E,F,
	appositive(false)])|R]):-
	getAppositive(L,R).
%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% getClass/2
%%
getClass([],[]):-!.
%% Word is an element name
getClass([posSent(M, Ui, Pos, Word, [A, B, C, P, N, H,
	W2])|X],
	[posSent(M, Ui, Pos, Word, [A, B, C, P, N, H, W2,
	class(Word)])|R]):-
	class_in_ontology(Word),
	getClass(X,R).

%% Word is a chemical element symbol
getClass([posSent(M, Ui, Pos, Symbol, [A, B, C, P, N, H,
	W2])|X],
	[posSent(M, Ui, Pos, Symbol, [A, B, C, P, N, H, W2,
	class(Word)])|R]):-
	abbrev_class_in_ontology(Symbol, Word),
	getClass(X,R).
getClass([posSent(M, Ui, Pos, Word, [A, B, C, P, N, H,
	W2])|X],
	[posSent(M, Ui, Pos, Word, [A, B, C, P, N, H, W2,
	class(none)])|R]):-
	getClass(X,R).
%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%
%%
%% getSuperClass/2
%%
%%
getSuperClass([],[]):-!.
%% Case 1: Word is listed in ontology as belonging to some
% element class
getSuperClass([posSent(M,Ui,Pos,Word,[A, B, C, P, N, H, W2,
	E])|X],
	[posSent(M,Ui,Pos,Word,[A, B, C, P, N, H,
	W2, E, superclass(Elementclass)])|R]):-
	class_superclass_in_ontology(Word, Elementclass),
	getSuperClass(X,R).
%% Case 2: Word is listed in ontology as a chemical symbol
getSuperClass([posSent(M,Ui,Pos,Word,[A, B, C, P, N, H, W2,
	E])|X],
	[posSent(M,Ui,Pos,Word,[A, B, C, P, N, H,
	W2, E, superclass(Elementclass)])|R]):-
        class_superclass_in_ontology(Class, Elementclass),
        abbrev_class_in_ontology(Word,Class),
	getSuperClass(X,R).
%% Case 3: Word itself is an element class
getSuperClass([posSent(M,Ui,Pos,Word,[A, B, C, P, N, H, W2,
	E])|X],
	[posSent(M,Ui,Pos,Word,[A, B, C, P, N, H,
	W2, E, superclass(Word)])|R]):-
	superclass_in_ontology(Word),
	getSuperClass(X,R).
%% Case 4: Word is listed as belonging to an element class by WordNet
getSuperClass([posSent(M,Ui,Pos,Word,[A, B, C, P, N, H, W2,
	E])|X],
	[posSent(M,Ui,Pos,Word,[A, B, C, P, N, H,
	W2, E, superclass(Elementclass)])|R]):-
	s(X,_,Word,_,_,_),
	hyp(X,Y),
	s(Y,_, Elementclass,_,_,_),
	superclass_in_ontology(Elementclass),
	getSuperClass(X,R).
%% Case 5: Word belongs to some element class, but which class is ambiguous
getSuperClass([posSent(M,Ui,Pos,Word,[A, B, C, P, N, H, W2,
	E])|X],
	[posSent(M,Ui,Pos,Word,[A, B, C, P, N, H,
	W2, E, superclass(amb)])|R]):-
	ambiguousclass(Word),
	getSuperClass(X,R).
%% Case 7: Word does not belong to any element class
getSuperClass([posSent(M,Ui,Pos,Word,[A, B, C, P, N, H, W2,
	E])|X],
	[posSent(M,Ui,Pos,Word,[A, B, C, P, N, H,
	W2, E, superclass(none)])|R]):-
	getSuperClass(X,R).
ambiguousclass('element').
%%
%%
%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%
%% changeCase/2
%%
%% Change case of all input words to lower case
%%
changeCase([],[]):-!.
changeCase([posSent(A,U,B,C)|D],[posSent(A,U,BLower,CLower)|R]):-
	lowercase(B,BLower),
	lowercase(C,CLower),
	changeCase(D,R).
changeCase([posSent(A,B,C)|D],[posSent(A,BLower,CLower)|R]):-
	lowercase(B,BLower),
	lowercase(C,CLower),
	changeCase(D,R).
%%
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% filterOut/2
%%
%% Remove periods and other symbols from input. Number all
% other words with a UniqueId
%% counter corresponding to word position in input text.

filterOut([],[]):-!.
filterOut([posSent(_,',',_)|A],B):- !,
	filterOut(A,B).
filterOut([posSent(_,'.',_)|A],B):- !,
	filterOut(A,B).
filterOut([posSent(_,'(',_)|A],B):- !,
	filterOut(A,B).
filterOut([posSent(_,'SYM',_)|A],B):- !,
	filterOut(A,B).
filterOut([posSent(A,B,C)|D],[posSent(A,UniqueId,B,C)|R]):-
	getCount(UniqueId),
	filterOut(D,R).
getCount(UniqueId):-
	count(UniqueId),
	retractall(count(_)),
	J is UniqueId + 1,
	assertz(count(J)).
%%
%%

%---------------------------------------------

%%%%%%%%%%%%%%%%%%%%%%%%%

%Appendix B
%Coreference Clustering Algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% semcluster.pl
%%
%% Take list of nouns phrase representations as input, calculate new
%% equivalence classes (initially each UniqueID is an equivalence
%% class)
%% and return list of modified equivalence classes
%%
%%
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% TOP LEVEL
%%

cluster(G,GG,Z):-
	buildRep(G,GG,V),
	%reverse(V,W),
	repeatalg(V,Y),
	getEqClass(Y,Z).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% repeatalg/2
%%
%% Recursively call clustering algorithm
%%
%%
repeatalg([],[]):- !.
repeatalg([I|J], [I|R]):-
	alg(I,J,J1),
	repeatalg(J1,R).
%%
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% alg/3
%%
%% Call algorithm on 1st and 2nd, then 1st and 3rd
%% elements of list
%%
alg(_,[], _):- !. %% stop when list of elements to compare with 1st is empty
alg(I,[J|K], [J1|R]):-
	distance_list(I,J,Sum),
	modify(Sum,I,J,[I|_],J1),
	alg(I,K,R).
%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Take distance(F), Np1(I), Np2(J) as input.
%% Return modified equivalence class(J1) if distance is
%% less than some threshold. J1 is J but with same UniqueID as I.
%% Where J1 is not modified, J1 is J unchanged.
%% Notes: remove unnecessary (SentNo, UniqueId, etc?)
%%
%% add var NpiClassList to alg...
modify(Sum, 
	posSent(M, UID, P, W, L),
	posSent(M2, UID2, P2, W2, L2), 
	NpiClassList,
        posSent(M2, UID, P,W2, L)):- 
	Sum < 4,
	all_nps_compatible(posSent(M, UID, P, W, L), 
		           posSent(M2, UID2, P2, W2, L2),
		           NpiClassList, 
                           _).

modify(Sum, posSent(_,_,_, _, _),
	posSent(SentNo, UniqueID, Pos, Word, List), 
	_,
	posSent(SentNo, UniqueID, Pos, Word, List)):-
	Sum >= 4. 

all_nps_compatible(Npi, Npj, NpiClassList, [NpiClassList|Npj]):-
	distance_list(Npi, Npj, Sum),
	Sum < 999.
%%
%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%%
distance_list(posSent([Max,_], _, _, Word, L1),
	      posSent([_,_], _, _, Word2, L2),
              Sum):-
	dist(L1,L2,Max,Word,Word2,List),
	accumulate_dist(List,Sum).
%%
%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% dist/6
%%
%% Calculate distances between NPs by calling
%% dist on attribute lists: match attribute lists for NPs
%% being compared; if attributes are different, check weight
%% and increment distance
%%
dist([],[],_,_,_,0):- !.
dist([F1|F1s], [F2|F2s], Max, Word,Word2,[D|Ds]):- %trace,
	d(F1,F2,Max,Word,Word2,D),
	dist(F1s,F2s,Max,Word,Word2,Ds).
d(singpl(none), singpl(none), _,_,_, 0).
d(singpl(none), singpl(_), _,_,_, 0).
%* \+ (X = none).
d(singpl(_), singpl(none), _,_,_, 0).
%* \+ (X = none).
d(singpl(Singpl), singpl(Singpl), _, _,_,0).
d(singpl(Singpl1), singpl(Singpl2),_, _,_,999):-
	\+ (Singpl1 = Singpl2).
d(article(none), article(none), _,_,_, 0).
d(article(none), article(_), _,_,_, 0).
d(article(_), article(none), _,_,_, 0).
d(article(Art), article(Art), _,_,_, 0).
d(article(Art1), article(Art2), _,_,_, 4):-
	\+ (Art1 = Art2).
d(pronountype(none), pronountype(none), _, _, _, 0).
% Different words and neither is a pronoun (eg fuses and
% systems)
d(pronountype(none), pronountype(_), _, _,_, -4).
% Different words and second is a pronoun (eg bismuth and
% it ): penalise cataphora
d(pronountype(_), pronountype(none), _,_,_, 4).
d(pronountype(X), pronountype(Y), _, _,_,999):-
	\+ (X = Y).
d(pronountype(X), pronountype(Y), _, _,_, -999):-
	Y = X.
d(name(none), name(none), _,_,_, 0).
d(name(none), name(_), _,_,_, 0).
d(name(_), name(none), _,_,_, 0).
d(name(X), name(X), _,_,_, -999).
d(name(X), name(Y),_, _,_, 999):-
	\+ (X = Y).
d(position(P), position(P2), Max,_,_, X):-
	P >= P2,
	X is ((P - P2)/Max) * 4.
d(position(P), position(P2), Max,_,_, X):-
	P < P2,
	X is ((P2 - P)/Max) * 4.
d(word(X), word(Y), _, _,_,4):-
	X = Y.
d(word(X), word(Y), _,_,_, 4):-
	\+ (X = Y).
d(appositive(false), appositive(false), _,_,_, 0).
d(appositive(false), appositive(true), _,_,_, -999).
d(appositive(true), appositive(false), _,_,_, 0).
d(appositive(true), appositive(true), _,_,_, 0).
d(class(none), class(none), _,_,_, 0).
d(class(none), class(_), _,_,_, 0).
d(class(_), class(none), _,_,_, 0).
%*\+(X = none).
d(class(X), class(Y), _, _,_, -999):-
	X = Y.
d(class(X), class(Y), _, _,_, 999):-
	\+ (X = Y).
d(superclass(none), superclass(none), _,_,_, 0).
d(superclass(none), superclass(_), _,_,_, 0).
d(superclass(_), superclass(none), _,_,_, 0).
d(superclass(amb), superclass(_), _,_,_, 0).
d(superclass(_), superclass(amb), _,_,_, 0).
d(superclass(X), superclass(Y), _,_,_, -4):-
	X = Y.
d(superclass(X), superclass(Y), _,_,_, 999):-
	\+ (X = Y).
%%
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% getEqClass/2
%%
%% Return only the equivalence class (Unique ID) and the word
%%
getEqClass([],[]):-!.
getEqClass([posSent([_,SentNo],Ui,_,Word,[_,_,_,_,_,_,_,_,_])|A],
	[class(SentNo,Ui,Word)|Uis]):-
	getEqClass(A,Uis).
%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% accumulate_dist/2
%%
%% add the contents of the distance list together
%%
accumulate_dist([], 0):- !.
accumulate_dist([Head|0], Sum):-
	Sum is Head + 0.
accumulate_dist([Head|Tail], Sum):-
	accumulate_dist(Tail, SubSum),
	Sum is SubSum + Head.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%---------------------------------------------
